#!/usr/bin/python -t
# -*- mode: Python; indent-tabs-mode: nil; -*-
#
# Retrieve name|summary|description for packages in src.rpm repo
# Author: Michael Schwendt <mschwendt@fedoraproject.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, sys

import yum, yum.Errors
from yum.misc import getCacheDir
from yum.packageSack import ListPackageSack


class RepoLister(yum.YumBase):
    def __init__(self, config = '/etc/yum.conf'):
        yum.YumBase.__init__(self)

        self.doConfigSetup(fn = config)

        if hasattr(self.repos, 'sqlite'):
            self.repos.sqlite = False
            self.repos._selectSackType()

    def readMetadata(self):
        self.doRepoSetup()
        self.doSackSetup(['src'])
        for repo in self.repos.listEnabled():
            self.repos.populateSack(which=[repo.id], mdtype='primary')

    def getPackageTuples(self):
        if yum.__version__ >= '2.9':  # TODO: check
            pkgs = self.pkgSack.returnNewestByName()
        else:
            pkgs = []
            for l in self.pkgSack.returnNewestByName():
                pkgs.extend(l)

        def sortbyname(a,b):
            return cmp(a.returnSimple('name'),b.returnSimple('name'))

        pkgs.sort(sortbyname)
        l = []
        for pkg in pkgs:
            name = pkg.returnSimple('name')
            summary = pkg.returnSimple('summary')
            description = pkg.returnSimple('description')
            l.append( (name,summary,description) )
        return l


def listPackageInfo(yumconf='/etc/yum.conf',repoids=[],cachedir=None):
    """Return list of tuples (name,summary,description) for all source rpms"""
    lister = RepoLister(config=yumconf)

    for repo in lister.repos.repos.values():
        if repo.id not in repoids:
            repo.disable()
        else:
            repo.enable()

    if os.geteuid() != 0 or cachedir:
        if not cachedir:
            cachedir = getCacheDir()
            if cachedir is None:
                print >> sys.stderr, "Error: Could not make cachedir, exiting"
                sys.exit(50)
        lister.repos.setCacheDir(cachedir)

    try:
        print "Reading repository metadata..."
        sys.stdout.flush()
        lister.readMetadata()
    except yum.Errors.RepoError, e:
        print >> sys.stderr, e
        raise

    return lister.getPackageTuples()


### testing only ###

if __name__ == "__main__":
    from optparse import OptionParser

    usage = "Usage: %s [options...]" % sys.argv[0]
    parser = OptionParser(usage=usage)
    parser.add_option("-c", "--config", default='/etc/yum.conf',
                      help='config file to use (defaults to /etc/yum.conf)')
    parser.add_option("-r", "--repoid", default=[], action='append',
                      help="specify repo ids to include, can be specified multiple times (default: disable all)")
    parser.add_option("-d", "--cachedir", default=None, 
                      help="specify a custom directory for metadata cache")
    (opts, args) = parser.parse_args()

    tuples = listPackageInfo(opts.config,opts.repoid,opts.cachedir)
    print "Packages:", len(tuples)
    for (name,summary,description) in tuples:
        print name, '  ', summary
    sys.exit(0)
